﻿package GameLib 
{
	import flash.events.Event;
	import flash.net.URLRequest;
	import flash.media.*;

	/**
	 * 簡易ゲームライブラリ：BGMクラス
	 * @author kazenetu
	 */
	public class BgmClass
	{
		private var channel : SoundChannel = null;
		private var bgm:Sound = null;
		
		private var _isPlay:Boolean = false;
		private var _isLoaded:Boolean = false;
		
		/**
		 * コンストラクタ
		 */
		public function BgmClass() 
		{
		}
		
		/**
		 * サウンドファイル（mp3）のロード
		 * @param	filePath 読み込むサウンドファイルのパス（相対パスもしくは絶対パス）
		 */
		public function load(filePath:String):void {
			//再生中であれば停止する
			if (this.channel != null) {
				this.channel.stop();
				this.channel = null;
			}
			//サウンドファイルが存在する場合はクローズする
			if(this.bgm != null){
				this.bgm.close();
				this.bgm = null;
			}
			//フラグをクリアする
			this._isPlay = false;
			this._isLoaded = false;
			
			//サウンドファイルをロードする
			this.bgm = new Sound(new URLRequest(filePath));
			this.bgm.addEventListener(Event.COMPLETE, this._loadComplete);
		}
		
		/**
		 * サウンドファイルのロード完了イベント
		 * @param	e Eventインスタンス
		 */
		private function _loadComplete(e:Event):void {
			//ロード完了イベントを削除
			this.bgm.removeEventListener(Event.COMPLETE, this._loadComplete);
			//ロード完了フラグをセット
			this._isLoaded = true;
		}
		
		/**
		 * サウンドファイルを再生する
		 * @param	startTime 開始位置（省略時=0 最初から）
		 * @param	loopCount 再生回数（省略時=100回）
		 * @param   volume ボリューム（省略時=1.0）
		 * @param	isMute true：音なし/false:音あり
		 */
		public function play(startTime:Number = 0, loopCount:int = 100, volume:Number=1.0, isMute:Boolean = false):void {
			if (this._isLoaded == false) {
				return;
			}

			if (isMute) {
				if (this.channel != null) {
					this.channel.stop();
					this.channel = null;
				}
			}
			else {
				if (this.channel == null) {
					this.channel = this.bgm.play(startTime,loopCount);
				}
				var trans:SoundTransform = channel.soundTransform;
				trans.volume = volume;
				this.channel.soundTransform = trans;
			}
			this._isPlay = true;
		}

		/**
		 * 再生中か、確認する
		 * @return true：再生中/false：停止中
		 */
		public function get isPlay():Boolean {
			return this._isPlay;
		}
		
		/**
		 * 再生中のサウンドファイルを停止する
		 */
		public function stop():void {
			if (this._isLoaded == false || this._isPlay==false) {
				return;
			}
			if (this.channel != null) {
				this.channel.stop();
				this.channel = null;
			}
			this._isPlay = false;
		}
		
		/**
		 * 解放処理
		 */
		public function dispose():void {
			if (this.channel != null) {
				this.channel.stop();
				this.channel = null;
			}
			if(this.bgm != null){
				this.bgm.close();
				this.bgm = null;
			}
		}

	}

}