﻿package GameLib 
{
	import flash.display.Stage;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Loader;
	import flash.events.Event;
	
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;

	import flash.net.URLRequest;
	import flash.system.LoaderContext;

	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.geom.ColorTransform;
	import flash.geom.Matrix;

	/**
	 * 簡易ゲームライブラリ：グラフィッククラス
	 * @author kazenetu
	 */
	public class GraphicClass
	{
		private var _refStage:Stage = null;
		private var _graphics:Object = null;
		private var _screen:Bitmap  = null;
		private var _backBuffer:BitmapData  = null;
		private var _tempBuffer:BitmapData  = null;
		private var _textField:TextField = null;
		
		/**
		 * コンストラクタ
		 * @param owner 呼び出し元のstageインスタンス
		 */
		public function GraphicClass(owner:Stage) 
		{
			this._refStage = owner;

			this._screen = new Bitmap(new BitmapData( this._refStage.stageWidth , this._refStage.stageHeight, false , 0xFFFFFF));
			this._backBuffer = new BitmapData( this._refStage.stageWidth , this._refStage.stageHeight, false , 0xFFFFFF);
			this._tempBuffer = new BitmapData( this._refStage.stageWidth , this._refStage.stageHeight, true , 0xFFFFFF);
			this._refStage.addChild(this._screen);
			
			this._textField = new TextField();
			this._textField.autoSize = TextFieldAutoSize.LEFT;
			
			this._graphics = new Object();
		}
		
		public function getGraphics():Object
		{
			return _graphics;
		}

		/**
		 * 画像ファイル（jpg,gif,png）の読み込み
		 * @param filePath 読み込む画像ファイルのパス（相対パスもしくは絶対パス）
		 */
		public function load(filePath:String):void {
			var ld:Loader = new Loader();
			ld.name = filePath;
			ld.load(new URLRequest(filePath));
			
			//ロード完了イベントを設定
			ld.contentLoaderInfo.addEventListener(Event.COMPLETE, this._onLoadComplete);

			//filePathをキーとして、_graphics連想配列にnullを追加する
			this._graphics[filePath] = null;
		}
		
		/**
		 * 画像ファイル（jpg,gif,png）の読み込み
		 * （filePathは読み込んだファイルのキーとなります）
		 * @param filePath 読み込む画像ファイルのパス（相対パスもしくは絶対パス）
		 */
		private function _onLoadComplete(event:Event) :void {
			var loader:Loader = event.target.loader as Loader;
			
			//BitmapDataを作成
			var srcBmpData:BitmapData = new BitmapData(loader.content.width,loader.content.height, true, 0x00000000);
			srcBmpData.draw(loader.content);
			
			//_graphics連想配列を更新する
			this._graphics[loader.name] = srcBmpData;
			
			//ロード完了イベントを削除
			loader.removeEventListener(Event.COMPLETE, this._onLoadComplete);
		}
		
		/**
		 * 画像ファイルが存在しているか、確認する。
		 * @param	filePath 画像ファイルを識別するキー（画像ファイルのパス（相対パスもしくは絶対パス））
		 * @return true：存在する（読み込みが完了していない場合も含む）/false:存在しない
		 */
		public function isExist(filePath:String):Boolean
		{
			return this._graphics.hasOwnProperty(filePath);
		}
		
		/**
		 * 画像ファイルが読み込み完了しているか、確認する。
		 * @param	filePath 画像ファイルを識別するキー（画像ファイルのパス（相対パスもしくは絶対パス））
		 * @return true：完了済/false:未完了、もしくは存在しない
		 */
		public function isLoaded(filePath:String):Boolean
		{
			if (this.isExist(filePath) && this._graphics[filePath]!=null) {
				return true;
			}
			return false;
		}
		
		/**
		 * ロード済みの画像ファイルをアンロードする
		 * @param	filePath 画像ファイルを識別するキー（画像ファイルのパス（相対パスもしくは絶対パス））
		 */
		public function unload(filePath:String):void {
			if (this.isExist(filePath)) {
				var bitmapData:BitmapData;
				if ((bitmapData = this._graphics[filePath]) != null) {
					bitmapData.dispose();
				}
				//解放
				delete this._graphics[filePath];
			}
		}
		
		/**
		 * バックバッファをクリアする
		 * ループ処理で描画する前に一度だけ実行してください。
		 * @param bgcolor 塗りつぶす色（省略時は白）
		 */
		public function clear(bgcolor:uint=0xFFFFFFFF):void {
			this._backBuffer.fillRect(new Rectangle(0, 0, this._backBuffer.width, this._backBuffer.height), bgcolor);
		}

		/**
		 * 画像ファイルをバックバッファに描画する
		 * @param	x 描画位置x
		 * @param	y 描画位置y
		 * @param	sx 画像ファイルの左上位置x
		 * @param	sy 画像ファイルの左上位置y
		 * @param	width 幅
		 * @param	height 高さ
		 * @param	filePath 画像ファイルを識別するキー（画像ファイルのパス（相対パスもしくは絶対パス））
		 */
		public function draw(x:Number, y:Number, sx:Number, sy:Number, width:Number, height:Number, filePath:String):void {
			//ロードが完了していない場合はそのまま終了
			if (this.isLoaded(filePath) == false) {
				return;
			}

			//バックバッファに描画する
			this._backBuffer.copyPixels(
				(this._graphics[filePath] as BitmapData)
				, new Rectangle(sx, sy, width, height)
				, new Point(x, y)
			);
		}

		/**
		 * 画像ファイルをバックバッファに描画する（拡大・縮小、回転対応版）
		 * 処理が重いので、等倍表示でよいのであれば、drawメソッドをご利用ください。
		 * @param	centerX 描画中心位置x（drawメソッドと異なり、画像の中心位置）
		 * @param	centerY 描画中心位置y（drawメソッドと異なり、画像の中心位置）
		 * @param	sx 画像ファイルの左上位置x
		 * @param	sy 画像ファイルの左上位置y
		 * @param	width 幅
		 * @param	height 高さ
		 * @param	filePath 画像ファイルを識別するキー（画像ファイルのパス（相対パスもしくは絶対パス））
		 * @param	scaleX X方向の拡大率（省略時は1.0）
		 * @param	scaleY Y方向の拡大率（省略時は1.0）
		 * @param	rad 回転ラジアン0.0～Math.PI*2（省略時は0.0）
		 * @param	color 表示カラー
		 */
		public function drawRich(centerX:Number, centerY:Number, sx:Number, sy:Number, width:Number, height:Number, filePath:String, 
								scaleX:Number=1, scaleY:Number=1,rad:Number=0,color:ColorTransform = null):void {
			if (this.isLoaded(filePath) == false) {
				return;
			}
			
			//一旦テンポラリバッファをクリアする
			this._tempBuffer.fillRect(new Rectangle(0, 0, this._tempBuffer.width, this._tempBuffer.height), 0xFFFFFF);
			
			//テンポラリバッファに等倍で中心位置に描画する
			this._tempBuffer.copyPixels(
				(this._graphics[filePath] as BitmapData)
				, new Rectangle(sx, sy, width, height)
				, new Point(
					(this._tempBuffer.width-width)/2
					,(this._tempBuffer.height-height)/2
				)
			);
			
			//テンポラリバッファをバックバッファに描画する
			var m:Matrix = new Matrix();
			m.scale(scaleX, scaleY);	//拡大・縮小
			m.translate(
				-this._tempBuffer.width/2*scaleX
				,-this._tempBuffer.height/2*scaleY);//回転のためにテンポラリバッファの中心位置を左上(0,0)に移動
			m.rotate(rad);	//回転
			m.translate(
				centerX
				,centerY);	//描画位置に移動
			this._backBuffer.draw(
				this._tempBuffer
				,m
				,color
			);
			
			return;
		}
		
		/**
		 * テキストを描画する
		 * @param	x 描画開始位置X
		 * @param	y 描画開始位置Y
		 * @param	text 描画する文字列
		 * @param	textColor 文字色（省略時は黒）
		 * @param	fontSize 文字サイズ（省略時は20ピクセル）
		 */
		public function drawText(x:int, y:int, text:String,textColor:uint=0x000000,fontSize:int=20):void {
			//TextFieldが生成されていなければ、終了
			if (this._textField == null) {
				return;
			}
			
			//TextFieldを設定
			this._textField.defaultTextFormat = new TextFormat("", fontSize, textColor);
			this._textField.text = text;
			
			var m:Matrix = new Matrix();
			m.translate(x, y);
			
			//バックバッファにテキストを出力
			this._backBuffer.draw(this._textField,m);
		}

		/**
		 * バックバッファをスクリーンに反映する
		 * ループ処理で描画した後に一度だけ実行してください。
		 */
		public function flip():void {
			this._screen.bitmapData.copyPixels(
				this._backBuffer 
				, new Rectangle(0, 0, this._backBuffer.width, this._backBuffer.height)
				, new Point(0, 0)
			);
		}
		
		/**
		 * 解放処理
		 * グラフィッククラスの利用が終わったら実行してください。
		 */
		public function dispose():void
		{
			var bitmapData:BitmapData;
			for (var name:String in this._graphics) {
				if ((bitmapData = this._graphics[name]) != null) {
					bitmapData.dispose();
				}
				delete this._graphics[name];
			}
			this._graphics = null;
			this._textField = null;
			this._refStage.removeChild(this._screen);
			this._screen.bitmapData.dispose();
			this._backBuffer.dispose();
			this._tempBuffer.dispose();
			this._refStage = null;
		}
		
	}

}