﻿package GameLib 
{
	import flash.display.Stage;
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;

	/**
	 * 簡易ゲームライブラリ：キーボード入力クラス
	 * @author kazenetu
	 */
	public class KeyClass
	{
		private var _refStage:Stage = null;
		private var _keys:Object = null;

		/**
		 * コンストラクタ
		 * @param owner 呼び出し元のstageインスタンス
		 */
		public function KeyClass(owner:Stage) 
		{
			this._refStage = owner;
			this._keys = new Object();
			
			//イベント登録
			this._refStage.addEventListener(KeyboardEvent.KEY_DOWN, this._onKeyDown);
			this._refStage.addEventListener(KeyboardEvent.KEY_UP, this._onKeyUp);
		}

		/**
		 * キーダウンイベント
		 * @param	e キーボードイベント
		 */
		private function _onKeyDown(e:KeyboardEvent):void
		{
			if (this._keys.hasOwnProperty(e.keyCode)) {
				this._keys[e.keyCode] = true;
			}
		}

		/**
		 * キーアップイベント
		 * @param	e キーボードイベント
		 */
		private function _onKeyUp(e:KeyboardEvent):void
		{
			if (this._keys.hasOwnProperty(e.keyCode)) {
				this._keys[e.keyCode] = false;
			}
		}

		/**
		 * 取得するキーコードを追加する
		 * @param	KeyCode flash.ui.Keyboardで指定するキーコード
		 */
		public function addKeyCode(KeyCode:uint):void
		{
			this._keys[KeyCode] = false;
		}
		
		/**
		 * 追加したキーコードを削除する
		 * @param	KeyCode flash.ui.Keyboardで指定するキーコード
		 */
		public function removeKeyCode(KeyCode:uint):void
		{
			if (this._keys.hasOwnProperty(KeyCode)) {
				delete this._keys[KeyCode];
			}
		}
		
		/**
		 * 指定したキーコードの状態を取得する
		 * @param	KeyCode flash.ui.Keyboardで指定するキーコード
		 * @return true:押している/false:押していない・キーコードが追加されていない
		 */
		public function getKeyState(KeyCode:uint):Boolean
		{
			if (this._keys.hasOwnProperty(KeyCode)) {
				return this._keys[KeyCode] as Boolean;
			}
			return false;
		}
		
		/**
		 * 解放処理
		 * キーボード入力クラスの利用が終わったら実行してください。
		 */
		public function dispose():void
		{
			//イベント削除
			this._refStage.removeEventListener(KeyboardEvent.KEY_DOWN, this._onKeyDown);
			this._refStage.removeEventListener(KeyboardEvent.KEY_UP, this._onKeyUp);

			this._keys = null;
			this._refStage = null;
		}
	}

}